#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Timestamp: "2025-12-23 09:55:10 (ywatanabe)"
# File: /home/ywatanabe/proj/figrecipe/src/figrecipe/PLOTTING_METHODS.py


"""Top-level docstring here"""

PLOTTING_METHODS = {
    "plot",
    "scatter",
    "bar",
    "barh",
    "hist",
    "hist2d",
    "boxplot",
    "violinplot",
    "pie",
    "errorbar",
    "fill",
    "fill_between",
    "fill_betweenx",
    "stackplot",
    "stem",
    "step",
    "imshow",
    "pcolor",
    "pcolormesh",
    "contour",
    "contourf",
    "quiver",
    "barbs",
    "streamplot",
    "hexbin",
    "tripcolor",
    "triplot",
    "tricontour",
    "tricontourf",
    "eventplot",
    "stairs",
    "ecdf",
    "matshow",
    "spy",
    "loglog",
    "semilogx",
    "semilogy",
    "acorr",
    "xcorr",
    "specgram",
    "psd",
    "csd",
    "cohere",
    "angle_spectrum",
    "magnitude_spectrum",
    "phase_spectrum",
    "graph",
}

# EOF
