#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Hitmap and selection JavaScript for the figure editor.

This module re-exports from the _hitmap subpackage.
See _hitmap/__init__.py for the full implementation.
"""

from ._hitmap import SCRIPTS_HITMAP

__all__ = ["SCRIPTS_HITMAP"]

# EOF
