#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Timestamp: "2026-02-12 11:01:03 (ywatanabe)"
# File: /home/ywatanabe/proj/figrecipe/examples/09c_diagram_shapes_out/hello.py

def hello():
    print("Hello, SciTeX!")

# EOF
