#!/usr/bin/env python3
"""figrecipe color utilities — RGB, RGBA, BGR, HEX conversions and palettes."""

from ._colormap import (
    get_categorical_colors_from_cmap,
    get_categorical_colors_from_conf_matap,
    get_color_from_cmap,
    get_color_from_conf_matap,
    get_colors_from_cmap,
    get_colors_from_conf_matap,
)
from ._colors import (
    bgr2bgra,
    bgr2rgb,
    bgra2bgr,
    bgra2hex,
    bgra2rgba,
    cycle_color,
    cycle_color_bgr,
    cycle_color_rgb,
    gradiate_color,
    gradiate_color_bgr,
    gradiate_color_bgra,
    gradiate_color_rgb,
    gradiate_color_rgba,
    rgb2bgr,
    rgb2rgba,
    rgba2bgra,
    rgba2hex,
    rgba2rgb,
    str2bgr,
    str2bgra,
    str2hex,
    str2rgb,
    str2rgba,
    to_hex,
    to_rgb,
    to_rgba,
    update_alpha,
)
from ._interpolate import gen_interpolate, interpolate
from ._PARAMS import DEF_ALPHA, HEX, PARAMS, RGB, RGB_NORM, RGBA, RGBA_NORM

__all__ = [
    "PARAMS",
    "DEF_ALPHA",
    "RGB",
    "RGB_NORM",
    "RGBA",
    "RGBA_NORM",
    "HEX",
    "bgr2bgra",
    "bgr2rgb",
    "bgra2bgr",
    "bgra2hex",
    "bgra2rgba",
    "cycle_color",
    "cycle_color_bgr",
    "cycle_color_rgb",
    "gen_interpolate",
    "get_categorical_colors_from_cmap",
    "get_categorical_colors_from_conf_matap",
    "get_color_from_cmap",
    "get_color_from_conf_matap",
    "get_colors_from_cmap",
    "get_colors_from_conf_matap",
    "gradiate_color",
    "gradiate_color_bgr",
    "gradiate_color_bgra",
    "gradiate_color_rgb",
    "gradiate_color_rgba",
    "interpolate",
    "rgb2bgr",
    "rgb2rgba",
    "rgba2bgra",
    "rgba2hex",
    "rgba2rgb",
    "str2bgr",
    "str2bgra",
    "str2hex",
    "str2rgb",
    "str2rgba",
    "to_hex",
    "to_rgb",
    "to_rgba",
    "update_alpha",
]

# EOF
