#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Alias for figrecipe.pyplot - drop-in replacement for matplotlib.pyplot.

Usage:
    import figrecipe.plt as plt  # Short form
    # or
    import figrecipe.pyplot as plt  # Full form
"""

from .pyplot import *  # noqa: F401, F403
