#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""figrecipe CLI entry point.

Allows running figrecipe as a module:
    python -m figrecipe <command>
"""

from ._cli import main

if __name__ == "__main__":
    main()
