#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""HTML components for the figure editor.

This package contains modular HTML components:
- file_browser: File browser panel
"""

from ._file_browser import HTML_FILE_BROWSER

__all__ = ["HTML_FILE_BROWSER"]

# EOF
